import os

DEFAULT_HEADERS = {
    # 'Accept-Language': 'en-US,en;q=0.9,vi;q=0.8',
    # 'Accept-Encoding': 'gzip, deflate, br',
    # 'Accept': 'text/html, */*; q=0.01',
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) '
                  'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.110 Safari/537.36',
}

CAPTCHA_API = os.getenv('CAPTCHA_API', 'http://captcha.lehongnam.com/')
LVS_AGENT_DOMAIN = os.getenv('LVS_AGENT_DOMAIN', 'https://edy688.com/')



"""
__version__.py
~~~~~~~~~~~~~~

Information about the current version of the py-package-template package.
"""

__title__ = 'cc-lvs'
__description__ = 'cc-lvs - LVS Agent API'
__version__ = '1.0.0'
__author__ = 'Cockroach Creative'
__author_email__ = 'support@cockreative.com'
__license__ = 'Apache 2.0'
__url__ = 'https://cockreative.com/'



import io
import random
import string
from urllib.parse import urlparse


import requests
from . import settings


def captcha_solver(img, **kwargs):
    if isinstance(img, str):
        img = io.StringIO(img)

    if isinstance(img, bytes):
        img = io.BytesIO(img)

    r = requests.post(settings.CAPTCHA_API, data=kwargs, files=dict(file=img))

    return r.text


def random_word(length):
    return ''.join(random.choice(string.ascii_lowercase) for i in range(length))


def encrypt_string(encryption_exponent, encryption_modulus, string_to_encrypt):
    return requests.get('https://ok368.lehongnam.com',
                        params=dict(expo=encryption_exponent, modulus=encryption_modulus, str=string_to_encrypt)
                        ).text


def get_uri(uri='', base=None):
    return (base or settings.LVS_AGENT_DOMAIN) + uri


def get_be_url(uri, base):
    return get_uri(uri, base.replace('ag.', 'be.'))


def get_base_url(origin):
    url = urlparse(origin)  # type: ParseResult
    return url.scheme + "://" + url.netloc + '/'



from collections import defaultdict

from bs4 import BeautifulSoup

from .utils import get_uri
from . import settings


def format_date(start):
    return start.strftime('%Y-%m-%d')


def format_float(text):
    return float(text.replace(',', ''))


def patch_session(session):
    session.headers.update(settings.DEFAULT_HEADERS)


def login(session, username, password, domain=None):
    # init request to create session
    # session.get(get_uri('login', domain))

    session.headers.update(settings.DEFAULT_HEADERS)

    if domain is None:
        domain = settings.LVS_AGENT_DOMAIN

    data = {
        'account': username,
        'passwd': password
    }

    # actual login request
    r = session.post('https://edy688.com/auth', params=data)

    return domain


def get_name_rank(html):
    soup = BeautifulSoup(html, 'html.parser')
    info = soup.find(attrs={'id': 'login-info'}).find_all('span')
    return [i.text.lower().split('sub')[0] for i in info]


def profile(session, domain):
    r = session.get(get_uri('', domain))
    return get_name_rank(r.text)


def parse_report(row):
    cols = row.find_all('td')

    username = cols[1].text.lower()

    yield username, 'turnover', format_float(cols[5].text)
    yield username, 'net_turnover', format_float(cols[12].text)
    yield username, 'commission', format_float(cols[7].text)
    yield username, 'win_lose', format_float(cols[9].text)


def parse_reports(html):
    try:
        soup = BeautifulSoup(html, 'html.parser')
        rows = soup.find('tbody').find_all('tr', attrs={'class': 'report-info'})

        for i in rows:
            yield from parse_report(i)

    except AttributeError:
        pass


def get_report(session, domain, start, end, root=None):
    if root is None:
        root, _ = profile(session, domain)

    r = session.get(get_uri('report/info', domain), params={
        'start': format_date(start),
        'end': format_date(end),
        'timetype': 0,
        'servo': ''
    }, headers={'x-requested-with': 'XMLHttpRequest'})

    base_data = defaultdict(int)

    for item in parse_reports(r.text):
        yield 'casino', *item
        if isinstance(item[2], (float, int)):
            base_data[item[1]] += item[2]

    for k, v in base_data.items():
        yield 'casino', root, k, v
